@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 
+import sys
 import time
 import traceback
 from multiprocessing import Process
@@ -7,11 +8,8 @@ from multiprocessing import Process
 import pytest
 import six
 
+from thriftpy2 import _compat
 from thriftpy2.thrift import TType, TPayloadMeta
-try:
-    from thriftpy2.protocol import cybin
-except ImportError:
-    cybin = None
 import thriftpy2
 from thriftpy2.http import (
     make_server as make_http_server,
@@ -27,6 +25,11 @@ from thriftpy2.rpc import make_server as make_rpc_server, \
     make_client as make_rpc_client
 from thriftpy2.transport import TBufferedTransportFactory, TCyMemoryBuffer
 
+
+if sys.platform == "win32":
+    pytest.skip("requires fork", allow_module_level=True)
+
+
 protocols = [TApacheJSONProtocolFactory,
              TJSONProtocolFactory,
              TBinaryProtocolFactory,
@@ -263,12 +266,12 @@ def test_complex_binary(proto_factory):
     time.sleep(0.2)
 
 
-@pytest.mark.skipif(cybin is None, reason="Must be run in cpython")
+@pytest.mark.skipif(_compat.PYPY, reason="Must be run in cpython")
 def test_complex_map():
     """
     Test from #156
     """
-    proto = cybin
+    from thriftpy2.protocol import cybin as proto
     b1 = TCyMemoryBuffer()
     proto.write_val(b1, TType.MAP, {"hello": "1"},
                     spec=(TType.STRING, TType.STRING))
@@ -279,7 +282,7 @@ def test_complex_map():
                     spec=(TType.STRING, TType.BINARY))
     b2.flush()
 
-    assert b1.getvalue() != b2.getvalue()
+    assert b1.getvalue() == b2.getvalue()
 
 
 type_map = {
