@@ -18,6 +18,10 @@ else:
     from thriftpy2.tornado import make_server
     from thriftpy2.transport import TTransportException
 
+    if sys.platform == "win32":
+        pytest.skip("add_socket is not implemented on Windiws",
+                    allow_module_level=True)
+
     logging.basicConfig(level=logging.INFO)
 
     addressbook = thriftpy2.load(path.join(path.dirname(__file__),
@@ -104,7 +108,6 @@ else:
             super(TornadoRPCTestCase, self).tearDown()
 
         @testing.gen_test
-        @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason="not support")
         def test_make_client(self):
             linus = addressbook.Person(name='Linus Torvalds')
             success = yield self.client_with_url.add(linus)
@@ -113,7 +116,6 @@ else:
             assert not success
 
         @testing.gen_test
-        @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason="not support")
         def test_synchronous_result(self):
             dennis = addressbook.Person(name='Dennis Ritchie')
             success = yield self.client.add(dennis)
@@ -124,7 +126,6 @@ else:
             assert person.name == dennis.name
 
         @testing.gen_test
-        @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason="not support")
         def test_synchronous_exception(self):
             exc = None
             try:
@@ -135,7 +136,6 @@ else:
             assert isinstance(exc, addressbook.PersonNotExistsError)
 
         @testing.gen_test
-        @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason="not support")
         def test_synchronous_undeclared_exception(self):
             exc = None
             try:
@@ -146,7 +146,6 @@ else:
             assert isinstance(exc, TTransportException)
 
         @testing.gen_test
-        @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason="not support")
         def test_asynchronous_result(self):
             dennis = addressbook.Person(name='Dennis Ritchie')
             yield self.client.add(dennis)
@@ -154,7 +153,6 @@ else:
             assert success
 
         @testing.gen_test
-        @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason="not support")
         def test_asynchronous_exception(self):
             exc = None
             try:
@@ -164,7 +162,6 @@ else:
             assert isinstance(exc, addressbook.PersonNotExistsError)
 
         @testing.gen_test
-        @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason="not support")
         def test_asynchronous_undeclared_exception(self):
             exc = None
             try:
