@@ -83,10 +83,16 @@ def write_field_stop(outbuf):
 
 
 def write_list_begin(outbuf, etype, size):
+    if etype == TType.BINARY:
+        etype = TType.STRING
     outbuf.write(pack_i8(etype) + pack_i32(size))
 
 
 def write_map_begin(outbuf, ktype, vtype, size):
+    if ktype == TType.BINARY:
+        ktype = TType.STRING
+    if vtype == TType.BINARY:
+        vtype = TType.STRING
     outbuf.write(pack_i8(ktype) + pack_i8(vtype) + pack_i32(size))
 
 
@@ -209,7 +215,8 @@ def read_map_begin(inbuf):
     return k_type, v_type, sz
 
 
-def read_val(inbuf, ttype, spec=None, decode_response=True):
+def read_val(inbuf, ttype, spec=None, decode_response=True,
+             strict_decode=False):
     if ttype == TType.BOOL:
         return bool(unpack_i8(inbuf.read(1)))
 
@@ -242,7 +249,8 @@ def read_val(inbuf, ttype, spec=None, decode_response=True):
             try:
                 return byte_payload.decode('utf-8')
             except UnicodeDecodeError:
-                pass
+                if strict_decode:
+                    raise
         return byte_payload
 
     elif ttype == TType.SET or ttype == TType.LIST:
@@ -261,7 +269,8 @@ def read_val(inbuf, ttype, spec=None, decode_response=True):
             return []
 
         for i in range(sz):
-            result.append(read_val(inbuf, v_type, v_spec, decode_response))
+            result.append(read_val(inbuf, v_type, v_spec, decode_response,
+                                   strict_decode))
         return result
 
     elif ttype == TType.MAP:
@@ -290,19 +299,21 @@ def read_val(inbuf, ttype, spec=None, decode_response=True):
             return {}
 
         for i in range(sz):
-            k_val = read_val(inbuf, k_type, k_spec, decode_response)
-            v_val = read_val(inbuf, v_type, v_spec, decode_response)
+            k_val = read_val(inbuf, k_type, k_spec, decode_response,
+                             strict_decode)
+            v_val = read_val(inbuf, v_type, v_spec, decode_response,
+                             strict_decode)
             result[k_val] = v_val
 
         return result
 
     elif ttype == TType.STRUCT:
         obj = spec()
-        read_struct(inbuf, obj, decode_response)
+        read_struct(inbuf, obj, decode_response, strict_decode)
         return obj
 
 
-def read_struct(inbuf, obj, decode_response=True):
+def read_struct(inbuf, obj, decode_response=True, strict_decode=False):
     while True:
         f_type, fid = read_field_begin(inbuf)
         if f_type == TType.STOP:
@@ -328,7 +339,8 @@ def read_struct(inbuf, obj, decode_response=True):
                 continue
 
         setattr(obj, f_name,
-                read_val(inbuf, f_type, f_container_spec, decode_response))
+                read_val(inbuf, f_type, f_container_spec, decode_response,
+                         strict_decode))
 
 
 def skip(inbuf, ftype):
@@ -374,11 +386,12 @@ class TBinaryProtocol(TProtocolBase):
 
     def __init__(self, trans,
                  strict_read=True, strict_write=True,
-                 decode_response=True):
+                 decode_response=True, strict_decode=False):
         TProtocolBase.__init__(self, trans)
         self.strict_read = strict_read
         self.strict_write = strict_write
         self.decode_response = decode_response
+        self.strict_decode = strict_decode
 
     def skip(self, ttype):
         skip(self.trans, ttype)
@@ -399,7 +412,8 @@ class TBinaryProtocol(TProtocolBase):
         pass
 
     def read_struct(self, obj):
-        return read_struct(self.trans, obj, self.decode_response)
+        return read_struct(self.trans, obj, self.decode_response,
+                           self.strict_decode)
 
     def write_struct(self, obj):
         write_val(self.trans, TType.STRUCT, obj)
@@ -407,12 +421,13 @@ class TBinaryProtocol(TProtocolBase):
 
 class TBinaryProtocolFactory(object):
     def __init__(self, strict_read=True, strict_write=True,
-                 decode_response=True):
+                 decode_response=True, strict_decode=False):
         self.strict_read = strict_read
         self.strict_write = strict_write
         self.decode_response = decode_response
+        self.strict_decode = strict_decode
 
     def get_protocol(self, trans):
         return TBinaryProtocol(trans,
                                self.strict_read, self.strict_write,
-                               self.decode_response)
+                               self.decode_response, self.strict_decode)
